/*
   author:  Rony G. Flatscher
   date:    2005-04-18/19/20
   changed: 2005-05-03, ---rgf, made sure that exceptions are propagated
            2005-06-05, ---rgf, uppercasing "OOo.cls" to "OOO.CLS", removing
            2005-07-03, ---rgf, changed logic to queryInterface(), now dealing with duplicate class names
                                gracefully; fixed a bug (do not wrap up BSF arrays)
            2005-07-04, ---rgf, optimized handling interface classes,
                                added OOo.getInterfaces(object[, sortByUnqualifiedClassName]),
                                added OOo.getServices(object)
            2005-07-05, ---rgf, removed unnecessary preloading of application classes
            2005-07-08, ---rgf, ooo.wrap(): now tries a bsf.wrap(), if ncecessary
            2005-07-30, ---rgf, adapted OOO.CLS to new array-behaviour in BSF.CLS
            2005-08-18, ---rgf, added public routines "ConvertToURL()", "ConvertFromURL()", which use
                                "encodeURL()", "decodeURL()"

   version: 0.2.5
   name:    OOo.cls
   purpose: ease interfacing with OpenOffice.org, v1.1.x and up
   needs:   BSF4Rexx
*/


   -- if no environment entry, then create the entry with default classes
if .local~hasentry("OOo")=.false then
do
   .local~OOo=.directory~new  -- create a new directory object and store it in .local

   -- preload some important classes, will become available by sending their class name to directory object ".ooo"
   l = .list~of( "com.sun.star.beans.PropertyValue"   , -
                 "com.sun.star.comp.helper.Bootstrap" , -
                 "com.sun.star.uno.Any"               , -
                 "com.sun.star.uno.AnyConverter"      , -
                 "com.sun.star.uno.UnoRuntime"       )

   do c over l -- get Java class object and save it in .OOo by className
      call OOo.loadClass c
   end

   .OOo~noProps=.bsf~bsf.createArray(.OOo~propertyValue, 0)   -- create empty array of property-value

   -- list of Interface classes to be loaded at runtime (needs completion)
   l = OOo.getInterfaceList()    -- retuns > 950 OOo-interface classes in a list

   -- save interface class names with which we want to deal
   d=.directory~new  -- directory for all InterfaceTypes
   r=.relation~new   -- relation to store duplicates (if last word is used multiple times, keep full class name)
   do i over l
     idx=substr(i, lastpos(".", i)+1)
     d~setentry(i, i)    -- by fully qualified name

     if d~hasentry(idx) then  -- interface class name used already, use relation to store full name
     do
        r[idx~translate]=i    -- save full name with class name
     end
     else   -- new interface class name
     do
       d~setentry(idx, i)     -- by last word
     end
   end
   .OOo~XInterfaces = d       -- save directory
   .OOo~XInterfaces.dupes = r -- save relation
end


::requires BSF.CLS                -- get BSF4Rexx support
::requires OOO_XINTERFACES.REX    -- get the list of all of OOo XInterface classes


      -- load OOo-class and save it in environment
::routine OOo.loadClass public
     parse arg className, idx

     if idx="" then idx=substr(className, lastpos(".", className)+1) -- extract last word to serve as index
     if .OOo~hasentry(idx)=.true then    -- e.g. last word is used as a class in a another package as well
        .error~say("---> OOo.loadClass():" pp(idx) "already used, in hand:" pp(className) "in .OOo:" pp(.OOo~entry(idx)))

-- say "className="className "idx="idx

-- rgf, 2005-07-29: use bsf.import(className, .nil) instead!
--     tmpClass=.OOo_proxy~new(.bsf~bsf.import(className, .nil))  -- make it look like an ooRexx class

     -- tmpClass=ooo.wrap(.bsf~bsf.import(className, .nil))  -- make it look like an ooRexx class

     tmpClass=.ooo_proxy~new(.bsf~bsf.import(className, .nil))  -- make it look like an ooRexx class

/*
say "ooo.cls: tmpClass=["tmpClass"]"
say
say "ooo.cls: idx="idx", tmpClass~id="tmpClass~id", tmpClass~bsf.fullJavaClassName=["tmpClass~bsf.fullJavaClassName"]."
*/

--     tmpClass=.OOo_proxy~new(.bsf4rexx~class.class~forName(className))  -- wrap it up

     .OOo~setentry(idx, tmpClass)   -- load the given Java class and save it in .OOo
     return tmpClass



      -- connects to given UNO server, returns the XMultiServiceFactory interface object of the remote service manager
::routine OOo.connect public
  parse arg unoURL
      -- no unoURL given, use default
 if unoURL="" then unoURL="uno:socket,host=localhost,port=8100;urp;StarOffice.NamingService"
--  if unoURL="" then unoURL="uno:socket,host=localhost,port=8200;urp;StarOffice.NamingService"

-- say "===> unoURL="pp(unoURL)

  context=.OOo~Bootstrap~createInitialComponentContext(.nil)
  uur= context~getServiceManager() ~createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", context)
-- say "OOO.CLS: context="context "- context~class="context~class
-- say "OOO.CLS: uur    ="uur     "- uur    ~class="uur    ~class

  remoteObject        = uur~XUnoUrlResolver ~resolve(unoURL)
  remoteNamingService = remoteObject~XNamingService

  remoteServiceManager=remoteNamingService~getRegisteredObject("StarOffice.ServiceManager")
  remoteMSF           = remoteServiceManager~XMultiServiceFactory
  if .OOo~hasentry("remoteMSF")=.false then .OOo~remoteMSF=remoteMSF -- save remote MultiServiceFactory
  return remoteMSF  -- return the remote XMultiServiceFactory



      -- get and return the XDesktop interface object
::routine OOo.getDesktop public
  USE ARG remoteMSF

  if remoteMSF="" | remoteMSF=.nil then remoteMSF=.OOo~remoteMSF -- use the stored remote MultiServiceFactory
  remoteDesktop=remoteMSF~createInstance("com.sun.star.frame.Desktop")
      -- return the XDesktop interface of the remote desktop
  return .OOo~unoRuntime~queryInterface(.OOo~XDesktop, remoteDesktop)



   -- modelled after VB's iif(), some people are so used to it that they must have it at all times
::routine iif public
  if arg(1)=.true then return arg(2)
                  else return arg(3)



   -- a routine for inserting values or formulas into cells
::routine OOo.setCell public
   use arg xSheet, x, y, content

	xSheet~getCellByPosition(x, y) ~setFormula(content)



/* *************************************************************
   Class:       OOo_PROXY
   Purpose:     allows to wrap up Java objects as Object Rexx objects;
                if instances are destroyed, then no derigstration from
                the BSF registry takes place
   restriction: needed to make getting interface objects easier
   ************************************************************* */
::CLASS OOo_PROXY -- subclass BSF PUBLIC

::Method bsfObject ATTRIBUTE

::METHOD init
   expose arg bsfObject -- direct access to BSF object
   use arg bsfObject    -- assign object to attribute

      -- set beanName
   self~objectName=bsfObject~objectName   -- use same objectname as BSF proxy (= idx into BSF-registry)

      -- if bsfObject is an imported Java class object, then use its beanName directly
   if bsfObject~string~left(4)~translate="THE " then
   do
-- say "bsfObject=[" || bsfObject"]"
      signal on syntax name continue   -- if an error, not a JCO, hence use its "plain" objectname
      self~objectname=.bsf~bsf.getJavaClassObjectProxy(bsfObject~bsf.FullJavaClassName)~string
   end
-- say "OOO.CLS ===> ===> self~objectName="self~objectname
   return

continue:   -- catch exception
/*
   -- default objectname
   self~objectName=bsfObject~objectName   -- use same objectname as BSF proxy (= idx into BSF-registry)
say "OOO.CLS ---> ---> self~objectName="self~objectname
*/


::METHOD unknown
   expose arg bsfObject -- direct access to BSF object
   parse upper arg xName 1 idx   -- make sure message name is in uppercase (if quoted)

   if pos(".", idx)>0 then idx=substr(idx, lastpos(".", idx)+1)   -- getindex

   SIGNAL ON SYNTAX NAME ANY  -- intercept all exceptions
   if idx~left(1)="X" then    -- if an interface object is asked for, query and return it
   do
      xClass=getInterfaceClass(xName, idx)

         -- query and return the interface object
      interface=.OOo~unoRuntime~queryInterface(xClass, bsfObject)
      if interface=.nil then  -- not found, maybe a duplicate simple class name ?
      do
say "ooo.cls: interface=.nil for" xName", trying relation..."
         do fullName over .ooo~XInterfaces.dupes~allAt(idx)
say "   idx="idx "fullName="fullName
            idx=substr(fullName, lastpos(".", fullName)+1)   -- getindex
            xClass=getInterfaceClass(xName, idx)

say "   xClass="|| xClass~string
            if xClass<>.nil then
               interface=.OOo~unoRuntime~queryInterface(xClass, bsfObject)

            if interface<>.nil then
               return .OOo_proxy~new(interface)
         end
say "ooo.cls: didn't work out, r~items:" .ooo~XInterfaces.dupes~allAt(idx)~items
         return .nil          -- no interface available
      end

      return .OOo_proxy~new(interface)
      -- return .OOo_proxy~new(.OOo~unoRuntime~queryInterface(xClass, bsfObject))
   end
   else  -- pass to BSF to handle it
   do
      if bsfObject~hasmethod(xName) then -- o.k. it's a BSF method (could be ooRexx OBJECT method as well!)
         FORWARD TO (bsfObject) MESSAGE (xName) ARGUMENTS (arg(2))
      else
         FORWARD TO (bsfObject) CONTINUE -- let BSF handle this

      tmpObj=result

      -- if a BSF-object, but not a BSF array: wrap it up
      if tmpObj~hasmethod("bsf.setFieldValue") then
      do
           -- if a BSF_ARRAY, then leave unchanged
         tmpClass=tmpObj~class~id   -- get class name
         if abbrev(tmpClass, "BSF_ARRAY") then return tmpObj   -- do not wrap up a BSF array object

           -- plain BSF object, hence wrap it up
         return .OOo_proxy~new(tmpObj)
      end

      return tmpObj
   end

ANY:
-- say "---> ---> ---> Ooops! OOO.CLS, SIGL="SIGL "condition()="condition()
  RAISE PROPAGATE    -- raise exception in caller

-- local function
-- try to get Interface class object; if not available, load it ("lazy loading")
getInterfaceClass: procedure
   parse arg xName, idx

   if xName<>idx then xClass=.OOo~entry(xName)  -- try to get interface class object, if already created
                 else xClass=.OOo~entry(idx)    -- try to get interface class object, if already created

   if xClass=.nil then        -- no entry as of yet
   do
      if xName<>idx then      -- long name given, then use that
      do
         xClass=OOo.loadClass(.ooo~XInterfaces~entry(xName), xName)
         if xClass<>.nil then -- o.k. also save with class name
         do
             .OOo~setEntry(idx, xClass)
-- say "long class name" pp(xName)"/"pp(idx)
         end
      end
      else     -- only class name is given
      do
         xClass=OOo.loadClass(.ooo~XInterfaces~entry(idx), idx)
         if xClass<>.nil then -- o.k., also save with full class name
         do
             .OOo~setEntry(.ooo~XInterfaces~entry(idx), xClass)
-- say "class name" pp(idx)"/"pp(.ooo~XInterfaces~entry(idx))
         end
      end
   end

   return xclass




  -- wrap argument into an OOo-proxy object, if it is a BSF object, else return argument unchanged
::routine OOo.wrap public
  if arg(1)~class=.OOo_proxy then return arg(1) -- already wrapped up

  if arg(1)~hasmethod("BSF.EXIT") then
     return .OOo_proxy~new(arg(1)) -- o.k. a BSF object

  o=bsf.wrap(arg(1))    -- hmm, maybe not yet wrapped up as a BSF-object
  if o~hasmethod("BSF.EXIT") then
     return .OOo_proxy~new(o) -- o.k. a BSF object

  return arg(1)                                 -- return argument unchanged


   -- pretty print (enclose argument in square brackets)
::routine pp public
  return "[" || arg(1)~string || "]"


   -- convert a file-name with path to URL notation as needed by OOo
::routine ConvertToUrl public    -- named after {Star|OOo}Basic
  parse arg str
  return (iif(left(str,1)="/", "file://", "file:///") || encodeURL(str))~changestr("\", "/")


   -- convert a file-name with path encoded in the URL notation to an operating system compliant file with path
::routine ConvertFromUrl public  -- named after {Star|OOo}Basic
  parse arg str

  new=decodeURL(str)             -- decode URL (replace escapes with the escaped chars themselves


  if left(new,5)="file:" then
  do
     dosLike= (substr(new, 10,1)=":")  -- if 10th char a colon, assume DOS-like filename

     if dosLike=.true then
        new=substr(new, 9)
     else      -- leave initial slash
        new=substr(new, 8)

     if .ooo~hasentry("FILE.SEPARATOR")=.false then   -- not yet in .OOo directory, hence add it
        .ooo~file.separator=.java.lang.system~getProperty("file.separator")   -- get info from Java

      -- if a DOS-like path and on a DOS-like system, translate slashes to backslashes
     if dosLike=.true | .ooo~file.separator="\" then
        return changestr("/", new, "\")
  end
  return new



::routine encodeURL public
  parse arg str
  new=changestr("%", str, "%25")    -- escape escape char %

   -- determine chars in URLs that do not get encoded
  url.chars="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/\?:=%'"".,+-"

  pos=verify(new, url.chars)     -- chars to escape in string?
  do while pos<>0
     char=substr(new, pos, 1)    -- extract char not in reference
     new=new~changestr(char, "%"||char~c2x)  -- escape character
     pos=verify(new, url.chars)
  end
  return new




   -- re-create chars from escape-sequences led in by % followed by two-digit hex value
::routine decodeUrl public    -- %20 --> " "
  parse arg str

  new=""
  do while str<>""
     parse var str before "%" +1 hexValue +2 str -- extract hex value
     new=new || before || hexValue~x2c
  end
  return new


/*
::routine nixi
-- the Java version does encode the first slash un Unix-style path-names !!
-- ::routine encodeUrl public    -- " " --> %20
  parse arg str

  signal on syntax
  -- charSet="US-ASCII"
  -- charSet="ISO-8859-1"
  -- return .java.net.URLEncoder~encode(str, charSet)
  return .java.net.URLEncoder~encode(str)

syntax:  -- if Java-URLEncoder not ready, then import it
  .bsf~bsf.import("java.net.URLEncoder")
say "---> URLEncoder got loaded..."
  -- return .java.net.URLEncoder~encode(str, charSet)
  return .java.net.URLEncoder~encode(str)


-- ::routine decodeUrl public    -- " " --> %20
  parse arg str

  signal on syntax
  charSet="US-ASCII"
  charSet="ISO-8859-1"
  return .java.net.URLDecoder~decode(str, charSet)
  -- return .java.net.URLDecoder~decode(str)

syntax:  -- if Java-URLEncoder not ready, then import it
  .bsf~bsf.import("java.net.URLDecoder")
say "---> URLDecoder got loaded..."
  return .java.net.URLDecoder~decode(str, charSet)
  -- return .java.net.URLDecoder~decode(str)




   -- returns an array, either sorted by full class name (default) or
   -- by unqualified class name, if second argument is given (any value)
::routine OOo.getInterfaces public
   use arg obj, sort

   if arg(2, "Omitted") then sort=1    -- if "1", sort by fully qualified name, else by unqualified class name

   if obj~class<>.OOo_proxy then
   do
     if \ obj~hasmethod("BSF.EXIT") then return .nil
     obj=.OOo_proxy~new(obj)  -- wrap up the BSF object
   end

   tp=obj~XTypeProvider    -- get type provider
   if sinfo=.nil then return .array~new   -- no interface ?
   ltp=tp~getTypes

   s=.set~new     -- use set to get rid of duplicate interfaces
   do t over ltp
      s~put(t~getTypeName)
   end

   tab="09"x      -- define tab character
   i=0
   do item over s
      if sort<>1 then
      do
         idx=substr(item, lastpos(".",item)+1)  -- extract simple class name
         item=idx||tab||item
      end
      i=i+1
      st.i=item
   end
   st.0=s~items

   call sysStemSort st. -- sort stem

   if sort=1 then return makeArrayFromStem(st.) -- return sorted stem

   do i=1 to st.0       -- remove simple class name
      parse var st.i . (tab) name
      st.i=name
   end
   return makeArrayFromStem(st.)


   -- returns a sorted array containing the implemented services
::routine OOo.getServices public
   use arg obj

   if obj~class<>.OOo_proxy then
   do
     if \ obj~hasmethod("BSF.EXIT") then return .nil
     obj=.OOo_proxy~new(obj)  -- wrap up the BSF object
   end

   sinfo=obj~XServiceInfo -- get service info interface
   if sinfo=.nil then return .array~new   -- no interface ?
   sn=sinfo~getSupportedServiceNames

   s=.set~new     -- use set to get rid of duplicate interfaces
   do service over sn
      s~put(service)
   end

   i=0
   do item over s
      i=i+1
      st.i=item
   end
   st.0=s~items

   call sysStemSort st. -- sort stem

   return makeArrayFromStem(st.) -- return sorted stem


/*
   -- returns a sorted array containing the implemented services
::routine OOo.getProperties public
   use arg obj

   if obj~class<>.OOo_proxy then
   do
     if \ obj~hasmethod("BSF.EXIT") then return .nil
     obj=.OOo_proxy~new(obj)  -- wrap up the BSF object
   end

   sinfo=obj~XPropertySetInfo -- get service info interface
   if sinfo=.nil then return .array~new   -- no interface ?
   sn=sinfo~getProperties

   s=.set~new     -- use set to get rid of duplicate interfaces
   do service over sn
      s~put(service~name)
   end

   i=0
   do item over s
      i=i+1
      st.i=item
   end
   st.0=s~items

   call sysStemSort st. -- sort stem

   return makeArrayFromStem(st.) -- return sorted stem
*/



-- stattdessen: introspection/refleciton !

   -- returns a sorted array containing the public methods of the class
::routine mist_OOo.getPublicMethods public
   use arg obj

-- say "obj="obj~string "class:" obj~class~id

   if obj~class<>.OOo_proxy then
   do
      if \ obj~hasmethod("BSF.EXIT") then return .nil
   end

      -- get the constant value for the PUBLIC modifier
   mPublic=bsf("getStaticValue", "java.lang.reflect.Modifier", "PUBLIC")
   numeric digits 10    -- make sure that doing arithmetics is o.k. in case of >2**31 as well
-- str= "mPublic="mPublic
   mPublic=d2c(mPublic) -- turn decimal number into binary representation
-- say str "nun:" mPublic~c2x    mPublic~c2x~x2b


   s=.set~new     -- use set to get rid of duplicate method names
   cl=obj~getClass   -- get Java class object: WRONG it's a Proxy-Class !!
   do m over cl~getDeclaredMethods -- obj~getClass~getDeclaredMethods
      if bitand(m~getModifiers~d2c, mPublic) <> "00"x then    -- a public method found
      do
         s~put(m~getName)
      end
   end

   i=0
   do item over s
      i=i+1
      st.i=item
   end
   st.0=s~items

   call sysStemSort st. -- sort stem

   return makeArrayFromStem(st.) -- return sorted stem


   -- local routine that turns the (sorted) stem into an array
::routine makeArrayFromStem
   use arg st.
   a=.array~new
   do i=1 to st.0
      a[i]=st.i
   end
   return a

*/
